<?/* dynwebstats, a dynamic PHP web statistics software for CLF log files
//@+leo-ver=4-thin
//@+node:111.20080423100525:@thin dynwebstats.php
//@@first
//@@comment //

dynwebstats is a web analytics software written in PHP. It parses a CLF (Common Log Format) log file (produced for instance by httpd or [[http://www.monperrus.net/martin/phpmodlog|phpmodlog]]) and outputs dynamically the corresponding web statistics.

Compared to its competitors:
    * dynwebstats outputs SVG graphics (viewable with good browsers like Firefox)
    * dynwebstats focuses on statistics **per URL**.

For feature requests or bug reports, [[http://www.monperrus.net/martin/|please drop me an email ]].

=====Screenshots=====
<a href="dynwebstats-graphical.png"><img width="600" src="dynwebstats-graphical.png" alt="screenshot 2"/></a><br/><br/>
<a href="dynwebstats-metrics.png"><img width="600" src="dynwebstats-metrics.png" alt="screenshot 2"/></a><br/><br/>
<a href="dynwebstats-others.png"><img width="600" src="dynwebstats-others.png" alt="screenshot 2"/></a>


=====Features=====
    * dynwebstats has a nice presentation of the statistics (see sreenshots below);
    * dynwebstats computes advanced metrics;
    * dynwebstats can compute statistics for 1 day, one month, one year, or what ever you want;
    * dynwebstats has a powerful referer analysis algorithm and presentation.

Compared to third party based solutions ([[http://www.google.com/analytics/|google analytics]], [[http://www.statcounter.com]], [[http://www.histats.com]], [[http://www.freestats.com]])
**Log files based software are better because you have**:
    * Total privacy: you and only you know who's going to your pages
    * Total control: You have a total control over your data
    * Total extensibility: You can implement any new metrics or views
    * Total logs: You get non javascript requests (e.g. mobile browsers, robots)

Compared to other log files based software ([[http://awstats.sourceforge.net/|awstats]], [[http://www.analog.cx|analog]], [[http://www.mrunix.net/webalizer/|webalizer]], [[http://www.hping.org/visitors/|visitors]], [[http://ftp.ics.uci.edu/pub/websoft/wwwstat/|wwwstat]], [[http://www.google.com/urchin/|urchin]]):
    * **dynwebstats computes reports per resource**
    * dynwebstats is dynamic (no cron required)
    * You get some very cool metrics (see below)

Compared to mysql based web analytics software ([[http://piwik.org/]] -- former phpmyvisites ), **dynwebstats is much more lightweight**.

**dynwebstats makes search traffic analysis** (same spirit as [[http://103bees.com]])
    * Total privacy on search words


===== Metrics =====

All metrics are given per resource or aggregated:

    * R: the number of regular requests (i.e. non robot) for a given resource
    * B: the number of robot-based requests
    * G: the number of googlebot-based requests
    * FG: the number of regular requests coming after a search on google
    * T: the number of hits
    * Eg: the efficiency (e-index) of google (see below). For instance a Eg of 12 means that google sends 12 people for one request by googlebot.
    * Dg: the google dependency (see index below). For instance, a Dg of 0.66 for a resource means that 66% of the traffic come from www.google.xx,. If the resource google ranking decreases, the overall traffic will also decrease, accordingly to the D-index.
    * Ig: the google interest (see below). For instance, a Ig of 100 means that googlebot visits the resource every 100 hours in average.

Referer based metrics:
    * [host] X: number of requests coming from host X
    * [searchword] Y: number of requests coming from search engines with search word Y
    * [searchexp] Z: number of requests coming from search engines with search expression Z
    * [url] T: number of requests coming from url T

Definitions:
    * the E-index is the efficiency index for a given robot. It is the ratio between the number of regular visitors comming from the search engine and the number of corresponding robot requests.
    * the D-index is the dependency index with respect to a search engine. It is the percentage of requests that come from the search engine.
    * the I-index is the interest index for a given robot. It is the average duration in hours between two requests of the robot.


===== Download =====

[[http://www.monperrus.net/martin/dynwebstats.zip]]

Simply create dynwebstats.local.php and add a line like the following (with the correct path to the webserver log):
&lt;? define('DYNWEBSTATS_DEFAULT_LOGFILENAME',"/var/log/apache/access.log"); ?>


===== Credits =====
dynwebstats uses ChartSVG and Joost's sortable-table




Copyright (C) 2006-2009 [[http://www.monperrus.net/martin/|Martin Monperrus]]
GNU General Public License


*/

/* TODO
 * create getters like getD1 etc.
 * integrate and delete lastaccesses => just one while loop
 * create a function to encapsulate link creation (and have short links)
 */

// why is dynwebstats not OO?

// are you authorized ? see http://www.monperrus.net/martin/simplephpauth
@include('simplephpauth.php');


//@+others
//@+node:111.20080423100842:Configuration

/** CONFIGURATION */

// USE THIS FILE TO MAKE YOUR OWN CHANGES
// THEN THE UPDATE WILL BE EASY: JUST REPLACING OLD dynwebstats.php by the new one
@include('dynwebstats.local.php');

//DYNWEBSTATS_DEFAULT_LOGFILENAME : the log file which must be writable by php/apache
@define('DYNWEBSTATS_DEFAULT_LOGFILENAME',"/var/log/httpd/access.log"); //ensure it is readable by php

// default: view from yesterday
@define('DYNWEBSTATS_DEFAULT_D1',360);

// default: view until now
@define('DYNWEBSTATS_DEFAULT_D2',0);

// number of points on the figure
@define('DYNWEBSTATS_DEFAULT_NPOINTS',12);

// threshold to avoid too long lines in tables
@define('DYNWEBSTATS_MAX_FIELD',70);

// threshold to appear in the statistics (remove noise)
@define('DYNWEBSTATS_FILE_THRESHOLD',3);

// threshold to appear in the referers
@define('DYNWEBSTATS_MAX_REFERER_LINES',50);

// number of last robot visits displayed
@define('DYNWEBSTATS_NUMROBOTVISITS',2);

// date format
@define('DYNWEBSTATS_DATEFORMAT','d/m/Y');

// names of robots
@define('DYNWEBSTATS_ROBOTS',"teoma|googlebot|webcrawler|fast|convera|biglotron|grub.org|slurp|UsineNouvelleCrawler|antibot|netresearchserver|nutch|ia_archiver|speedy|fluffy|msnbot|jyxobot|wget|curl|bibnum.bnf|findlink|exabot|gigabot|msrbot|seekbot|ngbot|scoutjet|libwww|panscient|yacybot|AISearchBot|IOI|ips-agent");

// User-Agent list
@define('DYNWEBSTATS_UA',"opera|netscape|safari|macintosh|msie|msie 4|msie 5|msie 6|msie 7|msie 8|windows|linux|apple|x11|firefox|firefox/2|firefox/3|konqueror|iceweasel");

// robot considered for i,g,r index
@define('DYNWEBSTATS_GOOGLE',"googlebot");

// size of the generated SVG images
@define('DYNWEBSTATS_IMG_SIZE','height="500" width="700"');

// charset
@define('DYNWEBSTATS_CHARSET','iso-8859-1');

// alias for the main metric: number of regular visitors
@define('DYNWEBSTATS_REGULAR','R');

// name of the GET parameter for the fake ALL file
@define('DYNWEBSTATS_ALL','__ALL__');

// default perl regexp pattern displayed in the summary view
@define('DYNWEBSTATS_DEFAULT_PATTERN','.*pdf');

@error_reporting(E_ALL);


//@-node:111.20080423100842:Configuration
//@+node:111.20080423100842.1:Functions
/** outputs "Powered by dynwebstats" enclosed in a DIV with the corresponding CSS style */
function poweredby() { ?>
<div style="margin:5px;text-align:right;opacity:0.6;font-size: xx-small;">Powered by <a href="http://www.monperrus.net/martin/dynwebstats">dynwebstats</a></div>
<? } // end function poweredby

/** takes a string representing a time in the CLF log format
 * and returns an integer representing the unix time
 */
function logtime_to_timestamp($time)
    {
        static $months = array(
            'Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun',
            'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec');

        $time_format = '/\[([\d]{2})\/([\w]{3})\/([\d]{4}):([\d]{2})'
            .':([\d]{2}):([\d]{2}) ([\+\-])([\d]{2})([\d]{2})\]/';

        $m = array();    //matches
        if(!preg_match($time_format, $time, $m)
                || count($m) != 10)
            return null;

        return
            mktime(
                $m[4], $m[5], $m[6],
                1 + array_search($m[2], $months), $m[1], $m[3]);
            //+ ($m[8] * 3600 + $m[9] * 60) * ($m[7] == '-' ? -1 : 1);

} // end method logtime_to_timestamp


/** computes the i-index for a given robot and the given data
 * and returns a real number
 */
function compute_x_index($robot,&$data)
{
    if (isset($data['last-bot-visits'][$robot])) {
      $nvisits= count($data['last-bot-visits'][$robot]);
      if ($nvisits>1) {
        $sum=0;
        $pred=0;
        foreach ($data['last-bot-visits'][$robot] as $k=>$item) {
                if ($k>0) {
                  @$sum+=($item-$pred);
                }
                $pred=$item;
        }
        $average_time = $sum/$nvisits;
        if ((time()-$pred) > $average_time )
        {
          $sum  = $sum+(time()-$pred);

          $nvisits++;
        }

        return @round($sum/($nvisits*3600));
      }
    }
    return "";

}

/** increments the value indexed by $key in the array &$tab */
function inc(&$tab,$key) {
  if (isset($tab[$key])) $tab[$key]++;
  else $tab[$key]=1;
}

/** increments by $v the value indexed by $key in the array &$tab */
function incv(&$tab,$key,$v) {
  if (isset($tab[$key])) $tab[$key]+=$v;
  else $tab[$key]=$v;
}

/** tests if a record is between two dates $d1 and $d2 (as unix times) */
function isConsidered(&$record,$d1,$d2) {
  return (($record['date']>$d1) && ($record['date']<$d2));
}

/** transforms a number of days from now ($t) to a unix time (integer) */
function deltaToDate($t) {
  return (time()-$t*3600*24);
}

/** parses a CLF line (for example from httpd's access.log)
 * and returns an array
 */
function clf2array($line) {

  $pattern = '/(.*) (.*) (.*) (\[.*\]) "(.*)" (.*) (.*) "(.*)" "(.*)"/U';

  if (!preg_match( $pattern, $line, $entries )) return null;

  $item = array();
  list( $complete_match, $item['ip'], $item['id'], $item['user'], $item['date-string'], $item['file'], $item['response-code'], $item['bytes-sent'], $item['Referer'], $item['User-Agent']  ) = $entries;
  $item['date'] = logtime_to_timestamp($item['date-string']);

  return $item;
}


/** maps a date as a unix time into an index for the web statistics barchart */
function date2index($unixtime) {
  $d1asdate = $_SESSION['d1u'];
  $timeframe=($_SESSION['d2u']-$d1asdate)/$_SESSION['n'];
  return floor(($unixtime-$d1asdate)/$timeframe);
 }

/** returns an XML file as a string for the URL $file
 * the XML is intended to be used with dynwebstats.xsl afterwards
 */
function xmlchart($file) {
$result="";
$result.='<?xml version="1.0" encoding="'.DYNWEBSTATS_CHARSET.'"?'.'>'."\n";

// WARNING: firefox requires a full link for XSL (security?)
// we don not use $_SERVER['SCRIPT_URL'] because it is not documented in php.net
$result.='<?xml-stylesheet type="text/xsl" href="http://'.$_SERVER['HTTP_HOST'].dirname($_SERVER['SCRIPT_NAME']).'/dynwebstats.xsl"?'.">\n";

$result.='<hsgraph id="hsgraph"><title></title><charttype>bar</charttype><showlegend chartfitpos="1"/>';
$result.='<xaxis>';
$result.='<max>'.$_SESSION['n'].'</max>';
$result.='<grid><longstrfit/>';
foreach($_SESSION['xaxis'] as $k => $v) $result.='<line x="'.$k.'">'.$v.'</line>';
$result.='</grid>';
$result.='</xaxis>';
$result.='<yaxis><max>'.getMaxY($file).'</max></yaxis>';
$result.='<series showvalues="">';
$result.='<serie><name>robots</name><color>#6699cc</color>';
foreach($_SESSION['xaxis'] as $k => $v) $result.='<point x="'.$k.'" y="'.@$_SESSION['byname'][$file]['periods'][$k]['B'].'"/>';
$result.='</serie>';

$result.='<serie><name>regular</name><color>#9966cc</color>';
foreach($_SESSION['xaxis'] as $k => $v) $result.='<point x="'.$k.'" y="'.@$_SESSION['byname'][$file]['periods'][$k]['R'].'"/>';
$result.='</serie>';

$result.='<serie><name>from google</name><color>#2266cc</color>';
foreach($_SESSION['xaxis'] as $k => $v) $result.='<point x="'.$k.'" y="'.@$_SESSION['byname'][$file]['periods'][$k]['FG'].'"/>';
$result.='</serie>';

$result.='</series>';
$result.='</hsgraph>';
return $result;
} // end xmlchart


/** gets the maximum value for $file
 * used for setting the barchart size
 */
function getMaxY($file) {
$max = 1;
foreach($_SESSION['byname'][$file]['periods'] as $point=>$v) {
   if (@$v[DYNWEBSTATS_REGULAR]>$max) $max = @$v[DYNWEBSTATS_REGULAR];
}

return $max*1.05;

}

/** is an enhanced urlencode with preg_quote */
function myurlencode($url) {
  return urlencode(preg_quote($url,'/'));
}


//@+node:martin.20090209102337.1:myurldecode
/** encapsulates the URL decoding for web stats */
function myurldecode($url) {
  return rawurldecode($url);
}
//@nonl
//@-node:martin.20090209102337.1:myurldecode
//@+node:111.20080429104744:setParameters
/** sets core parameters such as the log filename
 * the parameters are put in $_SESSION for historical reasons
 */
function setParameters() {


    if (isset($_GET['l'])&&file_exists($_GET['l'])) {
      $_SESSION['l']=$_GET['l'];
    }
    else {
       $_SESSION['l']=DYNWEBSTATS_DEFAULT_LOGFILENAME;
    }

    // d1
    if (isset($_GET['d1'])) {
       $_SESSION['d1']=$_GET['d1'];
    }
    else   {
     $_SESSION['d1']=DYNWEBSTATS_DEFAULT_D1;
    }



    // d2
    if (isset($_GET['d2'])) {
      $_SESSION['d2']=$_GET['d2'];

    }
    else  {
        $_SESSION['d2']=DYNWEBSTATS_DEFAULT_D2;
    }

    $_SESSION['d1u']=deltaToDate($_SESSION['d1']);
    $_SESSION['d2u']=deltaToDate($_SESSION['d2']);

   if ($_SESSION['d2u']<$_SESSION['d1u']) {die('error d2 greater than d1');}

  if (isset($_GET['n'])) {
      $_SESSION['n']=$_GET['n'];
      }
  else{
      $_SESSION['n']=DYNWEBSTATS_DEFAULT_NPOINTS;
  }

// x-axis legend
$xaxis=array();
$timeframe = ($_SESSION['d2u']-$_SESSION['d1u'])/$_SESSION['n'];
for($i=0;$i<$_SESSION['n'];$i++)
{
   $xaxis[$i]=date(DYNWEBSTATS_DATEFORMAT,$_SESSION['d1u']+$i*$timeframe+$timeframe/2);
}
$_SESSION['xaxis']=$xaxis;

$_SESSION['byname']=array();



}
//@-node:111.20080429104744:setParameters
//@+node:111.20080429103659:computeDerivedMetrics
/** computes various metrics based on $_SESSION['byname'][$file] */
function computeDerivedMetrics($file) {

     $data = $_SESSION['byname'][$file];

    // G metric
    $_SESSION['byname'][$file]['G']=@$data['byrobot'][DYNWEBSTATS_GOOGLE];

    // R metric
    foreach($data['periods'] as $point=>$v) {
      incv($_SESSION['byname'][$file],DYNWEBSTATS_REGULAR,@$v[DYNWEBSTATS_REGULAR]);
      incv($_SESSION['byname'][$file],'FG',@$v['FG']);
      incv($_SESSION['byname'][$file],'B',@$v['B']);
    }


 // i-index
    if (isset($data['last-bot-visits'])) {foreach($data['last-bot-visits'] as $bot => $value){
    $_SESSION['byname'][$file]['i-index'][$bot]=compute_x_index($bot,$data);
    }
    }
     $_SESSION['byname'][$file]['Ig']=@$_SESSION['byname'][$file]['i-index'][DYNWEBSTATS_GOOGLE];


   // e-index
    if (isset($_SESSION['byname'][$file]['G'])) $_SESSION['byname'][$file]['Eg']=round(@$_SESSION['byname'][$file]['FG']/$_SESSION['byname'][$file]['G']);

    // d-index
    if (isset($_SESSION['byname'][$file][DYNWEBSTATS_REGULAR])) $_SESSION['byname'][$file]['Dg']=round(@$_SESSION['byname'][$file]['FG']*100/$_SESSION['byname'][$file][DYNWEBSTATS_REGULAR]).'%';


    /*
    //this hangs some servers
    //I do not know why
    //the test is transfered in summary before printrow
    // when we track logs every day, we do not remove anything
    if ((!isset($data[DYNWEBSTATS_REGULAR]) || ($data[DYNWEBSTATS_REGULAR]<DYNWEBSTATS_FILE_THRESHOLD))) {
      unset($_SESSION['byname'][$file]);
    }
    */



}
//@-node:111.20080429103659:computeDerivedMetrics
//@+node:111.20080429101949:fastAnalysis
/** goes through the whole log file as fast as possible
 * and computes the simplest metric: number of hits
 */
function fastAnalysis() {

$_SESSION['fastResults']=array();
$logfilename = $_SESSION['l'];
$logfile = fopen($logfilename,'r');
$pattern = '/GET (.*) HTTP..... 200/U';
while ($line=fgets($logfile)) {
  if (preg_match( $pattern, $line, $entries )) {
    @$_SESSION['fastResults'][$entries[1]]++;
  }
}
fclose($logfile);

arsort($_SESSION['fastResults']);


}
//@nonl
//@-node:111.20080429101949:fastAnalysis
//@+node:111.20080429105930:resourceAnalysis
/** goes through the whole log file and matches the perl regexp $pattern
 * to gather base data
 * computeDerivedMetrics is called at th end to actually get the metrics
 */
function resourceAnalysis($pattern) {
// set default parameters in $_SESSION even if the session is disallowed
$logfilename = $_SESSION['l'];
$logfile = fopen($logfilename,'r');
// we only consider successfull HTTP requests (200)
$logpattern = '/GET ('.$pattern.') HTTP..... 200/U';
while ($line=fgets($logfile)) {
  if (preg_match( $logpattern, $line, $entries )) {
  $item = clf2array($line);
    if (isConsidered($item,$_SESSION['d1u'],$_SESSION['d2u'])) {
      recordAnalysis($item,$_SESSION['byname'][$pattern]);
      recordAnalysis($item,$_SESSION['byname'][$entries[1]]);
    }
  } // end if preg
} // end for each

foreach ($_SESSION['byname'] as $file => $data) {computeDerivedMetrics($file);}

}
//@-node:111.20080429105930:resourceAnalysis
//@+node:111.20080429101949.1:recordAnalysis
/** is the building block of analysis.
 * updates &$result based on the values of the array &$item */
function recordAnalysis(&$item,&$result) {

@$result['T']++;

// month treatment
  $month = date("Ym",$item['date']);

  // for the figure
  $point= date2index($item['date']);

  // by default we discard the entry
  $isregular = false;
  $isrobot = false;


  if (eregi(DYNWEBSTATS_ROBOTS,@$item['User-Agent'],$matches)) {
      $isrobot = true;
      $botname=strtolower($matches[0]);
      @$result['byrobot'][$botname]++;
      if (!isset($result['last-bot-visits'][$botname])) $result['last-bot-visits'][$botname] = array($item['date']);
      else {
        array_push($result['last-bot-visits'][$botname],$item['date']);
        }
    }

    $isregular =!$isrobot;

  if ($isregular) {
    @$result['periods'][$point][DYNWEBSTATS_REGULAR]++;

    // referer
    if ($item['Referer']!='')
    {

      $parsed_url=parse_url($item['Referer']);
      if (isset($parsed_url['host'])) {
        @$result['referer']['[host] '.$parsed_url['host']]++ ;
        // from google
        if(eregi("google\.",$parsed_url['host'])) {
          @$result['referer']['[host] google.xx (Dg)']++ ;
          @$result['periods'][$point]['FG']++;
        }

      }

      $search_phrase = get_search_phrase($item['Referer']);
      if (eregi('searchexp',$search_phrase)) {
          @$result['referer'][$search_phrase]++;
          $words = split("[[:space:]\"]",$search_phrase);
          foreach ($words as $w)
          {
                $w = strtolower($w);
                if (($w!="[searchexp]")&&($w!=""))  @$result['referer']['[searchword]  '.$w]++;

          }
        }
      else @$result['referer']['[url] '.($item['Referer'])]++;

    } // end isset(referer)
    else { /* we track possible new robots */ @$_SESSION['maybe_robot'][@$item['User-Agent']]++; }


    // user-agent treatment
    if ($item['User-Agent']!='') {
      foreach (explode("|", DYNWEBSTATS_UA) as $w ) {
      if (eregi($w,$item['User-Agent'])) {
        @$result['ua'][$w]++;
      }
      } // end foreach ($byuseragent as $w => $k)
    } // end isset($item['User-Agent']

  }

  if ($isrobot) { // is a robot
   @$result['periods'][$point]['B']++;
  }

} // end function recordAnalysis

//@-node:111.20080429101949.1:recordAnalysis
//@+node:111.20080429100732:get_search_phrase

/** returns the search phrase  prefixed by [searchexp] (if there is one)
* inspired from http://www.hackslash.net/?page_id=36 */
function get_search_phrase($referer){

    $search_phrase = "";
    $parsed_url=parse_url($referer);
    $rawquery = @$parsed_url['query'];

    // we could also use html_entity_decode
    $rawquery = str_replace("&amp;","&",$rawquery);


    $parsed_query = array();
    parse_str($rawquery,$parsed_query);

    if (isset($parsed_query['q'])) $search_phrase = $parsed_query['q'];
    if (($search_phrase == "") && isset($parsed_query['as_q'])) $search_phrase = $parsed_query['as_q'];
    if (($search_phrase == "") && isset($parsed_query['query'])) $search_phrase = $parsed_query['query'];
    if (($search_phrase == "") && isset($parsed_query['p'])) $search_phrase = $parsed_query['p'];

    if ($search_phrase != "")
    {
      if (get_magic_quotes_gpc()) $search_phrase= stripslashes($search_phrase);
      $search_phrase = utf8_decode(urldecode($search_phrase));
      $search_phrase = "[searchexp] ".strtolower($search_phrase);
      // google, position on search
      // pos has to be sticked to the number in order to give relevant info in the referer analysis
      if (isset($parsed_query['start'])) $search_phrase .= " (pos.".$parsed_query['start'].")";
    }

  return $search_phrase;
} // end get_search_phrase
//@-node:111.20080429100732:get_search_phrase
//@+node:111.20080429110522:printRow
/** prints a metric row of an HTML table for  $file */
function printRow($file,$headers=false) {

$metrics=array(DYNWEBSTATS_REGULAR,"Eg","Ig","Dg","B","G","FG","T");

if ($headers) {
echo '<tr><th>Resource</th>';
foreach($metrics as $x) echo "<th>{$x}</th>";
echo '</tr>';
return;
}
$total=@$_SESSION['byname'][DYNWEBSTATS_ALL][DYNWEBSTATS_REGULAR];

$reg=@$_SESSION['byname'][$file][DYNWEBSTATS_REGULAR];

    echo '<tr>';
    echo '<td><a href="?byfile&amp;'.'f='.myurlencode($file,'/').'&amp;d1='.$_SESSION['d1'].'&amp;d2='.$_SESSION['d2'].'&amp;l='.$_SESSION['l'].'">'.(strlen($file)>DYNWEBSTATS_MAX_FIELD?substr(htmlentities($file),-DYNWEBSTATS_MAX_FIELD):htmlentities($file)).'</a>';// we use htmlentities to protect ourself from javascript injection in the url
    echo "</td>";
   foreach($metrics as $x) echo "<td>".@$_SESSION['byname'][$file][$x]."</td>";
    echo "</tr>\n";

}
//@-node:111.20080429110522:printRow
//@+node:martin.20090304094039.1:zip
/** is used to deliver a "packaged" dynwebstats to the users who request a download */
function zip() {

header('Content-Type: application/zip');
system('zip  - dynwebstats.css  dynwebstats.js  dynwebstats.php  dynwebstats.xsl dynwebstats.local.php');


}
/** prints the main statistics menu as a DIV  (e.g. week report | month report)
 * the URLs contained embed all the necessary information */
function print_menu($file) {
    ?>
<div class="menu">
<a href="?byfile&amp;d1=7&amp;n=7&amp;f=<?=urlencode($file)?>&amp;l=<?=$_SESSION['l']?>">week report</a> |
<a href="?byfile&amp;d1=30&amp;n=30&amp;f=<?=urlencode($file)?>&amp;l=<?=$_SESSION['l']?>">month report</a> |
<a href="?byfile&amp;d1=91&amp;n=12&amp;f=<?=urlencode($file)?>&amp;l=<?=$_SESSION['l']?>">quarter report</a> |
<a href="?byfile&amp;d1=182&amp;n=12&amp;f=<?=urlencode($file)?>&amp;l=<?=$_SESSION['l']?>">semester report</a> |
<a href="?byfile&amp;d1=365&amp;n=12&amp;f=<?=urlencode($file)?>&amp;l=<?=$_SESSION['l']?>">year report</a> |
<a href="?byfile&amp;d1=1080&amp;n=36&amp;f=<?=urlencode($file)?>&amp;l=<?=$_SESSION['l']?>">3 years report</a> |
<a href="?lastaccesses&amp;d=1&amp;pattern=<?=urlencode($file)?>&amp;l=<?=$_SESSION['l']?>">last 24h records</a> |
<a href="?menu&amp;l=<?=$_SESSION['l']?>">menu</a>
</div>
<?
} // end print_menu


//@-node:martin.20090304094039.1:zip
//@-node:111.20080423100842.1:Functions
//@+node:martin.20081209092803.1:views


//@+node:111.20080423100842.3:menu (default view)
/** outputs the "Summarw" view, mainly a big HTML table (number of hits -> URL)
 * is a full HTML page
 */
function menu() {
// fast analysis of the log file
// for the menu and enable the fast mode if needed
fastAnalysis();
?><!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
   "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
<meta http-equiv="Content-Type" content="text/html; charset=<?=DYNWEBSTATS_CHARSET?>"/>
<link href="dynwebstats.css" rel="stylesheet" type="text/css" />
<meta name="robots" content="noindex"/></head>


<body>
<h1>URLs in <?=$_SESSION['l']?></h1>
<table>
<?

foreach ($_SESSION['fastResults'] as $file => $num) {
    if ($num > DYNWEBSTATS_FILE_THRESHOLD) {
    // use of htmlentities to protect ourself from javascript injection in the url
    echo '<tr><td>'.$num.'</td><td><a href="?byfile&amp;f='.myurlencode($file,'/').'&amp;d1='.$_SESSION['d1'].'&amp;d2='.$_SESSION['d2'].'&amp;l='.$_SESSION['l'].'">'.htmlentities($file)."</a></td></tr>\n";
    } // end if
}

?>

<tr><td></td><td>
<form action="?">
perl regexp:<input type="text" name="f" size="20" value="<?=DYNWEBSTATS_DEFAULT_PATTERN?>"/>
<input type="hidden" name="byfile" value="1"/>
<input type="hidden" name="l" value="<?=$_SESSION['l']?>"/>
<input type="hidden" size="2" name="d1" value="<?=$_SESSION['d1']?>"/>
<input type="hidden" size="2"  name="d2" value="<?=$_SESSION['d2']?>"/>
<input type="hidden" size="2"  name="n" value="<?=$_SESSION['n']?>"/>
<input type="submit"/>
</form>
</td></tr>

</table>

<h2>Tailor</h2>
<form action="?">
<table>
<tr><td>Log file name:</td><td><input type="text" name="l" value="<?=$_SESSION['l']?>"/></td></tr>
<tr><td>From:</td><td><input type="text" size="2" name="d1" value="<?=$_SESSION['d1']?>"/>days from today.</td></tr>
<tr><td>To:</td><td><input type="text" size="2"  name="d2" value="<?=$_SESSION['d2']?>"/>days from today.</td></tr>
<tr><td>Number of points:</td><td><input type="text" size="2"  name="n" value="<?=$_SESSION['n']?>"/></td></tr>
<tr><td></td><td><input type="submit"/></td></tr>
</table>
</form>


<?=poweredby()?>
</body>
</html>
<? }

//@-node:111.20080423100842.3:menu (default view)
//@+node:111.20080423100842.4:byfile
/** outputs the "metric" view for $_GET['f']
 * it is a full HTML page
 */
function byfile() {

// precondition
if (!isset($_GET['f'])) {
  echo "parameter required"; exit;
}


// stripslahes is really required
$file=stripslashes($_GET['f']);
//$file=$_GET['f'];

resourceAnalysis($file);

$title=ereg_replace('\\\\','',$file);


function byfile_menu($file) {?>
<a href="#gra">Graphical Report</a> |
<a href="#sum">Metrics</a> |
<a href="#rob">Robots Report</a> |
<a href="#use">User-agent Report</a> |
<a href="?lastaccesses&amp;d=<?=$_SESSION['d1']?>&amp;pattern=<?=urlencode($file)?>&amp;l=<?=$_SESSION['l']?>">Records</a>
<br/><br/>
<? } // end function byfile_menu

$div = isset($_SESSION['byname'][$file][DYNWEBSTATS_REGULAR])?$_SESSION['byname'][$file][DYNWEBSTATS_REGULAR]:100000000;

// important $data is used beloz for rbot, user agent analysis, etc.
$data=@$_SESSION['byname'][$file];

?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
   "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
<meta http-equiv="Content-Type" content="text/html; charset=<?=DYNWEBSTATS_CHARSET?>"/>
<meta name="generator" content="dynwebstats" />
<link href="dynwebstats.css" rel="stylesheet" type="text/css" />



<script type="text/javascript" src="dynwebstats.js"></script>
<title>Statistics for <?=$title?></title>
</head>
<body>
<? print_menu($file); ?>
<h1>Web Statistics for <?=$title?></h1>
Analysed requests from <?=date('r',$_SESSION['d1u'])?> to <?=date('r',$_SESSION['d2u'])?>.


<?
//if (!ereg('MSIE',@$_SERVER['HTTP_USER_AGENT'])) {
?>
<a name="gra"></a>
<h2>Graphical Report</h2>
<?=byfile_menu($file)?>
<object <?=DYNWEBSTATS_IMG_SIZE?> data="data:image/svg+xml;base64,<?=base64_encode(xmlchart($file))?>"></object>
<?
//}
?>

<a name="sum"></a>
<h2>Metrics</h2>
<?=byfile_menu($file)?>
<table class="sortable" id="sortabletable">
<?
printRow($file,true);
// when using .* as pattern we get all possible entries
if (count($_SESSION['byname']) > 2) {
    foreach ($_SESSION['byname'] as $url => $tmp) {printRow($url);}
}
// when using a fully qualified name, there is only two lines (one with antislashed regexp characters)
else printRow(array_pop(array_keys($_SESSION['byname'])));

?>
</table>


<? if (isset($data['referer'])) { ?>
<a name="referer"></a>
<h2>Referer Report</h2>
           <table>
          <tr><th>Referer</th><th>#</th><th>%reg</th></tr>
          <?


            arsort($data['referer']);
            foreach ($data['referer'] as $ref => $num) {
            if (@$nref++>DYNWEBSTATS_MAX_REFERER_LINES) break;
            echo '<tr>';
      //onmouseover="this.oldbgcolor=this.style.backgroundColor;this.onmouseout=function(){this.style.backgroundColor=this.oldbgcolor;};this.style.backgroundColor='darkblue';"
            //echo "<td onmouseover="this.childNodes[0].olddata=this.childNodes[0].data;this.onmouseout=function(){this.childNodes[0].data=this.childNodes[0].olddata;};this.childNodes[0].data='toto';">".($ref)."</td>";
            echo "<td>".htmlentities(substr($ref,0,DYNWEBSTATS_MAX_FIELD))."</td>";
            echo "<td>{$num}</td>";
            echo "<td>".@(round($num*100/$div))."%</td>";
            echo "</tr>\n";
            } // end foreach
            echo '</table>';
	} // end if (isset($data['referer']))
?>



<? if (isset($data['byrobot'])) { ?>
<a name="rob"></a>
<h2>Robots Report</h2>
<?=byfile_menu($file)?>

<table  class="sortable" id="sortabletable2">
<tr><th>Robots</th><th>#</th><th>%robots</th><th>%total</th><th>I</th><th>Last Visit</th></tr>
<?

arsort($data['byrobot']);
foreach ($data['byrobot'] as $bot=> $num) {
echo '<tr>';
echo "<td>{$bot}</td>";
echo "<td>{$num}</td>";
echo "<td>".round(($num/$data['B'])*100)."%</td>";
echo "<td>".round($num/($data['B']+@$data['R'])*100)."%</td>";
echo "<td>".@$data['i-index'][$bot]."</td>";
echo "<td>".date(DYNWEBSTATS_DATEFORMAT,@$data['last-bot-visits'][$bot][count(@$data['last-bot-visits'][$bot])-1])."</td>";

echo "</tr>\n";
}
?>
</table>


<? } // end  if (isset($data['byrobot'])) { ?>

<a name="use"></a>
<h2>User-agent Report</h2>
<?=byfile_menu($file)?>

<table>
<tr><th>User-agent</th><th>#</th><th>%reg</th></tr>
<?

if (isset($data['ua'])) {
arsort($data['ua']);
foreach ($data['ua'] as $file => $num) {
echo '<tr>';
echo "<td>{$file}</td>";
echo "<td>{$num}</td>";
echo "<td>".round(($num/$div)*100)."%</td>";
echo '</tr>';
}
}
?>
</table>

<?=poweredby()?>
</body>
</html>
<? } // end function byfile
//@-node:111.20080423100842.4:byfile
//@+node:111.20080423100842.5:chart
/** outputs the XML file corresponding to the barchart for $_GET['f'] */
function chart() {
// precondition
if (!isset($_GET['f'])) {
  echo "parameter required"; exit;
}
$file=stripslashes($_GET['f']);
resourceAnalysis($file);

xmlchart($file);
}
//@-node:111.20080423100842.5:chart
//@+node:111.20080425091951:lastaccesses
/** outputs the last records for $_GET['pattern']
 * from $_GET['d'] days until now */
function lastaccesses() {

// stripslashes is required since pattern is a regexp
$pattern = "/ ".stripslashes($_GET['pattern'])." /";

if (isset($_GET['d']))  $d=deltaToDate($_GET['d']);

else { echo '<B>you must specify the number of days (GET parameter d)</B>'; exit ;}

if ($d<deltaToDate(1)) $dateformat=DYNWEBSTATS_DATEFORMAT;

else $dateformat='H:i';


?>



<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
   "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head>

<meta http-equiv="Content-Type" content="text/html; charset=<?=DYNWEBSTATS_CHARSET?>"/>
<link href="dynwebstats.css" rel="stylesheet" type="text/css" />
<title>Last logs for <?=$pattern?></title>
<meta name="robots" content="noindex"/>
<!-- pattern = <?=$pattern?> -->
</head>
<body>
<? print_menu(stripslashes($_GET['pattern'])); ?>

<h1>Accesses of <?=stripslashes($_GET['pattern'])?> the <?=$_GET['d']?> last day(s)</h1>
<table>
<tr><th>Date</th><th>Host</th><th>Resource</th><th>Referer</th></tr>
<?
$logfile = fopen($_SESSION['l'],'r');

$i=1;
while ($line=fgets($logfile)) {
  $item = clf2array($line);
  if (isConsidered($item,$d,time())) {

    if (preg_match($pattern,$item["file"])&&!eregi(str_replace(',','|',DYNWEBSTATS_ROBOTS),@$item["User-Agent"])) {

      $x=array('host'=>'','ref'=>'');
      $x['date']=date($dateformat,$item['date']);
      if (ereg('^[0-9.]+$',$item["ip"])) $x['host']=gethostbyaddr($item["ip"]);

      else  $x['host']=@$item["ip"];

      if(isset($item['Referer'])) {
        $sp = get_search_phrase($item['Referer']);
        if ($sp == "")  $x['ref']=$item['Referer'];
        else $x['ref']=$sp;
      }

      echo($i%2)?'<tr>':'<tr class="tralternate">';
      echo "<td>".$x['date']."</td>";

      // substr is used for keeping short row
      echo "<td>".substr(htmlentities($x['host']),0,30)."</td>";

      // substr is used for removing GET .... HTTP/1.1
      echo "<td>".substr($item["file"],4,-9)."</td>";

      echo "<td>".htmlentities($x['ref'])."</td></tr>";
      $i++;

    }
  }
}
fclose($logfile);
?>
</table>

<?=poweredby()?>
</body>
</html>



<? }


//@-node:111.20080425091951:lastaccesses
//@-node:martin.20081209092803.1:views
//@+node:martin.20090304090849.1:main
/** is the dispatcher based on the URL query */
function main() {
// for historical reasons, $_SESSION is used for saving global variables
// for simplifying dynwebstat protocal and maintenance
// we do not use anymore session_start
//session_start();
setParameters();

foreach ($_GET as $key => $value) {
  if (function_exists($key)) {
    call_user_func($key);
  }
}

// if nothing happened ?
// links in headers must not contain &amp;
if (!headers_sent()) header('Location: '.$_SERVER['SCRIPT_NAME'].'?menu&l='.$_SESSION['l']);
}

// is dynwebstats used directly ?
if (__FILE__==$_SERVER['SCRIPT_FILENAME']) main();
// otherwise it is used as library

//@-node:martin.20090304090849.1:main
//@-others
//@@last
//@-node:111.20080423100525:@thin dynwebstats.php
//@-leo
?>